@@ -1,8 +1,9 @@
 import _ from 'lodash';
 import {ClientDelegate} from '../../lib/client_delegate';
+import kbn from 'app/core/utils/kbn';
 
 export class FlowDatasource {
-
+  /** @ngInject */
   constructor(instanceSettings, $q, backendSrv, templateSrv) {
     this.type = instanceSettings.type;
     this.url = instanceSettings.url;
@@ -16,7 +17,6 @@ export class FlowDatasource {
   query(options) {
     let start = options.range.from.valueOf();
     let end = options.range.to.valueOf();
-    let step = Math.floor((end - start) / options.maxDataPoints);
 
     if (options.targets.length > 1) {
       throw new Error("Multiple targets are not currently supported when using the OpenNMS Flow Datasource.");
@@ -36,39 +36,115 @@ export class FlowDatasource {
     // Filter
     let exporterNode = this.getFunctionParameterOrDefault(target, 'withExporterNode', 0);
     let ifIndex = this.getFunctionParameterOrDefault(target, 'withIfIndex', 0);
+    let applications = this.getFunctionParametersOrDefault(target, 'withApplication', 0, null);
+    let conversations = this.getFunctionParametersOrDefault(target, 'withConversation', 0, null);
+    let hosts = this.getFunctionParametersOrDefault(target, 'withHost', 0, null);
     // Transform
     let asTableSummary = FlowDatasource.isFunctionPresent(target, 'asTableSummary');
 
-    if (target.metric === 'conversations') {
-      if (!asTableSummary) {
-        return this.client.getSeriesForTopNConversations(N, start, end, step, exporterNode, ifIndex).then(series => {
-          return {
-            data: FlowDatasource.toSeries(target, series)
-          };
-        });
-      } else {
-        return this.client.getSummaryForTopNConversations(N, start, end, exporterNode, ifIndex).then(table => {
-          return {
-            data: FlowDatasource.toTable(table)
-          };
-        });
-      }
-    } else if (target.metric === 'applications') {
-      if (!asTableSummary) {
-        return this.client.getSeriesForTopNApplications(N, start, end, step, includeOther, exporterNode, ifIndex).then(series => {
-          return {
-            data: FlowDatasource.toSeries(target, series)
-          };
-        });
-      } else {
-        return this.client.getSummaryForTopNApplications(N, start, end, includeOther, exporterNode, ifIndex).then(table => {
-          return {
-            data: FlowDatasource.toTable(table)
-          };
-        });
-      }
+    // If a group by interval has been set we will use that to determine the step value, otherwise we will use the step
+    // value from Grafana's automatically calculated maxDataPoints (based on pixel width)
+    let groupByInterval = this.getFunctionParameterOrDefault(target, 'withGroupByInterval', 0, null);
+    let step;
+    if (groupByInterval) {
+      step = kbn.interval_to_ms(groupByInterval);
     } else {
-      throw 'Unsupported target metric: ' + target.metric;
+      step = Math.floor((end - start) / options.maxDataPoints);
+    }
+
+    switch (target.metric) {
+      case 'conversations':
+        if (!asTableSummary) {
+          if (conversations && conversations.length > 0) {
+            return this.client.getSeriesForConversations(conversations, start, end, step, includeOther, exporterNode, ifIndex).then(series => {
+              return {
+                data: FlowDatasource.toSeries(target, series)
+              };
+            });
+          } else {
+            return this.client.getSeriesForTopNConversations(N, start, end, step, includeOther, exporterNode, ifIndex).then(series => {
+              return {
+                data: FlowDatasource.toSeries(target, series)
+              };
+            });
+          }
+        } else {
+          if (conversations && conversations.length > 0) {
+            return this.client.getSummaryForConversations(conversations, start, end, includeOther, exporterNode, ifIndex).then(table => {
+              return {
+                data: FlowDatasource.toTable(target, table)
+              };
+            });
+          } else {
+            return this.client.getSummaryForTopNConversations(N, start, end, includeOther, exporterNode, ifIndex).then(table => {
+              return {
+                data: FlowDatasource.toTable(target, table)
+              };
+            });
+          }
+        }
+      case 'applications':
+        if (!asTableSummary) {
+          if (applications && applications.length > 0) {
+            return this.client.getSeriesForApplications(applications, start, end, step, includeOther, exporterNode, ifIndex).then(series => {
+              return {
+                data: FlowDatasource.toSeries(target, series)
+              };
+            });
+          } else {
+            return this.client.getSeriesForTopNApplications(N, start, end, step, includeOther, exporterNode, ifIndex).then(series => {
+              return {
+                data: FlowDatasource.toSeries(target, series)
+              };
+            });
+          }
+        } else {
+          if (applications && applications.length > 0) {
+            return this.client.getSummaryForApplications(applications, start, end, includeOther, exporterNode, ifIndex).then(table => {
+              return {
+                data: FlowDatasource.toTable(target, table)
+              };
+            });
+          } else {
+            return this.client.getSummaryForTopNApplications(N, start, end, includeOther, exporterNode, ifIndex).then(table => {
+              return {
+                data: FlowDatasource.toTable(target, table)
+              };
+            });
+          }
+        }
+      case 'hosts':
+        if (!asTableSummary) {
+          if (hosts && hosts.length > 0) {
+            return this.client.getSeriesForHosts(hosts, start, end, step, includeOther, exporterNode, ifIndex).then(series => {
+              return {
+                data: FlowDatasource.toSeries(target, series)
+              };
+            });
+          } else {
+            return this.client.getSeriesForTopNHosts(N, start, end, step, includeOther, exporterNode, ifIndex).then(series => {
+              return {
+                data: FlowDatasource.toSeries(target, series)
+              };
+            });
+          }
+        } else {
+          if (hosts && hosts.length > 0) {
+            return this.client.getSummaryForHosts(hosts, start, end, includeOther, exporterNode, ifIndex).then(table => {
+              return {
+                data: FlowDatasource.toTable(target, table)
+              };
+            });
+          } else {
+            return this.client.getSummaryForTopNHosts(N, start, end, includeOther, exporterNode, ifIndex).then(table => {
+              return {
+                data: FlowDatasource.toTable(target, table)
+              };
+            });
+          }
+        }
+      default:
+        throw 'Unsupported target metric: ' + target.metric;
     }
   }
 
@@ -149,7 +225,22 @@ export class FlowDatasource {
     });
   }
 
-  static toTable(table) {
+  static toTable(target, table) {
+    let toBits = FlowDatasource.isFunctionPresent(target, 'toBits');
+
+    if (toBits) {
+      let inIndex = table.headers.indexOf('Bytes In');
+      let outIndex = table.headers.indexOf('Bytes Out');
+      table.rows = _.map(table.rows, (row) => {
+        row[inIndex] *= 8;
+        row[outIndex] *= 8;
+        return row;
+      });
+      table.headers[inIndex] = 'Bits In';
+      table.headers[outIndex] = 'Bits Out';
+    }
+
+
     let columns = table && table.headers ? _.map(table.headers, column => {
       return {"text": column}
     }) : [];
@@ -159,6 +250,10 @@ export class FlowDatasource {
         "columns": columns,
         "rows": table.rows,
         "type": "table",
+        "meta": {
+          metric: target.metric,
+          toBits: toBits
+        }
       }
     ];
   }
@@ -274,6 +369,13 @@ export class FlowDatasource {
     return matchingFunctions.length > 0 ? matchingFunctions[0] : null;
   }
 
+  static getFunctions(target, name) {
+    let matchingFunctions = _.filter(target.functions, function (f) {
+      return f.name === name;
+    });
+    return matchingFunctions.length > 0 ? matchingFunctions : null;
+  }
+
   static isFunctionPresent(target, name) {
     return FlowDatasource.getFirstFunction(target, name) !== null;
   }
@@ -284,7 +386,30 @@ export class FlowDatasource {
       // No match, use the default value
       return def;
     }
+
     // Return the parameter value, and perform any required template variable substitutions
-    return this.templateSrv.replace(func.parameters[idx]);
+    if (_.isString(func.parameters[idx])) {
+      return this.templateSrv.replace(func.parameters[idx]);
+    } else {
+      return func.parameters[idx];
+    }
+  }
+
+  getFunctionParametersOrDefault(target, name, idx, def) {
+    let funcs = FlowDatasource.getFunctions(target, name);
+    if (funcs === null) {
+      // No match, use the default value
+      return def;
+    }
+
+    let returnFuncs = [];
+    funcs.forEach((func) => {
+      if (func.parameters[idx]) {
+        returnFuncs.push(this.templateSrv.replace(func.parameters[idx]));
+      }
+    });
+
+    // Return the parameter value, and perform any required template variable substitutions
+    return returnFuncs;
   }
 }
