@@ -1,10 +1,10 @@
 import {QueryType} from './constants';
 import {interpolate} from "./interpolate";
 import _ from 'lodash';
-import {FunctionFormatter} from './function_formatter';
+import {FunctionFormatter} from '../../lib/function_formatter';
 
 export class OpenNMSDatasource {
-
+  /** @ngInject */
   constructor(instanceSettings, $q, backendSrv, templateSrv) {
     this.type = instanceSettings.type;
     this.url = instanceSettings.url;
@@ -86,7 +86,7 @@ export class OpenNMSDatasource {
       });
     } else {
       // There are no sources listed, let Grafana display "No data points" to the user
-      return {'data': []};
+      return this.$q.when({ data: [] });
     }
 
     return request
@@ -396,10 +396,15 @@ export class OpenNMSDatasource {
           }
 
           value = columns[i].values[j];
+          // Replace literal 'NaN' values with nulls
+          if (value === 'NaN') {
+            value = null;
+          }
+
           if (!atLeastOneNonNaNValue && !isNaN(value)) {
             atLeastOneNonNaNValue = true;
           }
-          datapoints.push([columns[i].values[j], timestamps[j]]);
+          datapoints.push([value, timestamps[j]]);
         }
 
         let label = labels[i];
@@ -449,11 +454,12 @@ export class OpenNMSDatasource {
     return OpenNMSDatasource.getNodeResource(nodeId) + "." + resourceId;
   }
 
-  searchForNodes(query) {
+  searchForNodes(query, offset) {
     return this.doOpenNMSRequest({
       url: '/rest/nodes',
       method: 'GET',
       params: {
+        offset: offset,
         limit: this.searchLimit,
         match: 'any',
         comparator: 'ilike',
