@@ -5,7 +5,7 @@ import appEvents from 'app/core/app_events';
 import _ from 'lodash';
 
 export class OpenNMSQueryCtrl extends QueryCtrl {
-
+  /** @ngInject */
   constructor($rootScope, $scope, $injector, $q, $modal) {
     super($scope, $injector);
 
@@ -24,9 +24,9 @@ export class OpenNMSQueryCtrl extends QueryCtrl {
       'Label': 'label',
       'Foreign ID': 'foreignId',
       'sysName': 'sysName'
-    }, function (query) {
+    }, function (query, offset) {
       return self.datasource
-        .searchForNodes(query)
+        .searchForNodes(query, offset)
         .then(function (results) {
           return {
             'count': results.data.count,
@@ -59,13 +59,9 @@ export class OpenNMSQueryCtrl extends QueryCtrl {
         });
       }
 
-      // Limit the results - it takes along time to render if there are too many
-      var totalCount = filteredResources.length;
-      filteredResources = _.take(filteredResources, self.datasource.searchLimit);
-
+      // Passing All Filtered Resources to Modal - Pagination will be applied
       return {
-        'count': filteredResources.length,
-        'totalCount': totalCount,
+        'totalCount': filteredResources.length,
         'rows': filteredResources
       };
     }
@@ -122,7 +118,6 @@ export class OpenNMSQueryCtrl extends QueryCtrl {
           });
 
           return {
-            'count': namedAttributes.length,
             'totalCount': namedAttributes.length,
             'rows': namedAttributes
           };
@@ -144,7 +139,6 @@ export class OpenNMSQueryCtrl extends QueryCtrl {
         .getAvailableFilters()
         .then(function (results) {
           return {
-            'count': results.data.length,
             'totalCount': results.data.length,
             'rows': results.data
           };
@@ -190,6 +184,7 @@ export class OpenNMSQueryCtrl extends QueryCtrl {
   }
 
   validateTarget(targetId, required) {
+    // console.log('validateTarget(' + targetId + ',' + required + ')', this.target);
     if (this.target.type === QueryType.Attribute || this.target.type === QueryType.Expression) {
       var messages = {
         'nodeId': "You must supply a node id.",
@@ -205,9 +200,11 @@ export class OpenNMSQueryCtrl extends QueryCtrl {
         return targetId + ' is a required field.';
       }
     } else if (this.target.type === QueryType.Filter) {
-      if (targetId == 'filterName' && (!this.target.filter || !this.target.filter.name)) {
-        return "You must select a filter.";
-      } else if (required && (!this.target.filterParameters || !(targetId in this.target.filterParameters) || !this.target.filterParameters[targetId])) {
+      if (targetId === 'filterName') {
+        if (!this.target.filter || this.target.filter && (!this.target.filter.name || this.target.filter.name.trim().length === 0)) {
+          return "You must select a filter.";
+        }
+      } else if (targetId !== 'type' && required && (!this.target.filterParameters || !(targetId in this.target.filterParameters) || !this.target.filterParameters[targetId])) {
         return targetId + ' is a required field.';
       }
     }
