@@ -1,10 +1,10 @@
-import {API, Client, Rest} from '../opennms';
+import {API, Client, Rest} from 'opennms';
 import _ from 'lodash';
 
 let Q;
 
 export class ClientDelegate {
-
+    /** @ngInject */
     constructor(settings, backendSrv, $q) {
         this.type = settings.type;
         this.url = settings.url;
@@ -32,10 +32,9 @@ export class ClientDelegate {
           authConfig = new API.OnmsAuthConfig(username, password);
         }
 
-        let server = new API.OnmsServer(this.name, this.url, authConfig);
-        let http = new Rest.GrafanaHTTP(this.backendSrv, server, this.timeout);
+        const server = API.OnmsServer.newBuilder(this.url).setName(this.name).setAuth(authConfig).build();
+        const http = new Rest.GrafanaHTTP(this.backendSrv, server, this.timeout);
         this.client = new Client(http);
-        this.client.server = server;
         this.clientWithMetadata = undefined;
      }
 
@@ -70,14 +69,20 @@ export class ClientDelegate {
 
     getClientWithMetadata() {
         if (!this.clientWithMetadata) {
-              let self = this;
-              let client = Client.getMetadata(self.client.server, self.client.http, self.timeout)
+              const self = this;
+              const http = self.client.http;
+              const client = Client.getMetadata(http.server, http, self.timeout)
                 .then(function(metadata) {
                     // Ensure the OpenNMS we are talking to is compatible
-                    if (metadata.apiVersion() !== 2) {
+                    if (metadata.apiVersion() < 2) {
                         throw new Error("Unsupported Version");
                     }
-                    self.client.server.metadata = metadata;
+                    const server = API.OnmsServer.newBuilder(http.server.url)
+                        .setName(http.server.name)
+                        .setAuth(http.server.auth)
+                        .setMetadata(metadata)
+                        .build();
+                    http.server = server;
                     return self.client;
                 }).catch(function(e) {
                     // in case of error, reset the client, otherwise
@@ -88,13 +93,67 @@ export class ClientDelegate {
 
           // Grafana functions that invoke the datasource expect the
           // promise to be one that is returned by $q.
-          let deferred = this.$q.defer();
+          const deferred = this.$q.defer();
           client.then((success) => deferred.resolve(success)).catch((error) => deferred.reject(error));
           this.clientWithMetadata = deferred.promise;
         }
         return this.clientWithMetadata;
     }
 
+    // Inventory (node) related functions
+
+    getNodeDao() {
+        return this.getClientWithMetadata().then(function(client) {
+            return client.nodes();
+        });
+    }
+
+    findNodes(filter) {
+        return this.getNodeDao()
+            .then(function(nodeDao) {
+                return nodeDao.find(filter);
+            }).catch(this.decorateError);
+    }
+
+    getNode(nodeId) {
+      return this.getNodeDao()
+        .then(function(nodeDao) {
+            return nodeDao.get(nodeId);
+        }).catch(this.decorateError);
+    }
+
+    getNodeProperties() {
+        return this.getNodeDao()
+            .then(nodeDao => {
+                return nodeDao.searchProperties();
+            }).catch(this.decorateError);
+    }
+
+    findNodeProperty(propertyId) {
+        return this.getNodeProperties()
+            .then(properties => {
+                return _.find(properties, function(property) {
+                    return property.id === propertyId;
+                });
+            });
+    }
+
+    getNodePropertyComparators(propertyId) {
+        return this.findNodeProperty(propertyId)
+            .then(property => {
+                if (property) {
+                    const comparators = property.type.getComparators();
+                    if (comparators && comparators.length > 0) {
+                        return comparators;
+                    }
+                }
+                console.log("No comparators found for property with id '" + propertyId + "'. Falling back to EQ.");
+                // This may be the case when the user entered a property, which does not exist
+                // therefore fallback to EQ
+                return [ API.Comparators.EQ ];
+            }).catch(this.decorateError);
+    }
+
     // Fault related functions
 
     getAlarmDao() {
@@ -195,15 +254,15 @@ export class ClientDelegate {
         return this.$q.when(operators);
     }
 
-    getProperties() {
+    getAlarmProperties() {
         return this.getAlarmDao()
             .then(alarmDao => {
                 return alarmDao.searchProperties();
             }).catch(this.decorateError);
     }
 
-    findProperty(propertyId) {
-        return this.getProperties()
+    findAlarmProperty(propertyId) {
+        return this.getAlarmProperties()
             .then(properties => {
                 return _.find(properties, function(property) {
                     return property.id === propertyId;
@@ -211,8 +270,8 @@ export class ClientDelegate {
             });
     }
 
-    getPropertyComparators(propertyId) {
-        return this.findProperty(propertyId)
+    getAlarmPropertyComparators(propertyId) {
+        return this.findAlarmProperty(propertyId)
             .then(property => {
                 if (property) {
                     const comparators = property.type.getComparators();
@@ -258,6 +317,13 @@ export class ClientDelegate {
         }).catch(this.decorateError);
     }
 
+    getApplications(prefix, start, end, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getApplications(prefix, start, end, nodeCriteria, interfaceId);
+            }).catch(this.decorateError);
+    }
+
     getSeriesForTopNApplications(N, start, end, step, includeOther, nodeCriteria, interfaceId) {
         return this.getFlowDao()
             .then(function(flowDao) {
@@ -265,10 +331,10 @@ export class ClientDelegate {
             }).catch(this.decorateError);
     }
 
-    getSeriesForTopNConversations(N, start, end, step, nodeCriteria, interfaceId) {
+    getSeriesForApplications(applications, start, end, step, includeOther, nodeCriteria, interfaceId) {
         return this.getFlowDao()
             .then(function(flowDao) {
-                return flowDao.getSeriesForTopNConversations(N, start, end, step, nodeCriteria, interfaceId);
+                return flowDao.getSeriesForApplications(applications, start, end, step, includeOther, nodeCriteria, interfaceId);
             }).catch(this.decorateError);
     }
 
@@ -279,10 +345,88 @@ export class ClientDelegate {
             }).catch(this.decorateError);
     }
 
-    getSummaryForTopNConversations(N, start, end, nodeCriteria, interfaceId) {
+    getSummaryForApplications(applications, start, end, includeOther, nodeCriteria, interfaceId) {
         return this.getFlowDao()
             .then(function(flowDao) {
-                return flowDao.getSummaryForTopNConversations(N, start, end, nodeCriteria, interfaceId);
+                return flowDao.getSummaryForApplications(applications, start, end, includeOther, nodeCriteria, interfaceId);
+            }).catch(this.decorateError);
+    }
+
+    getSeriesForTopNConversations(N, start, end, step, includeOther, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getSeriesForTopNConversations({
+                    N: N,
+                    start: start,
+                    end: end,
+                    step: step,
+                    exporterNode: nodeCriteria,
+                    ifIndex: interfaceId,
+                    includeOther: includeOther
+                });
+            }).catch(this.decorateError);
+    }
+
+    getSeriesForConversations(conversations, start, end, step, includeOther, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getSeriesForConversations(conversations, start, end, step, includeOther, nodeCriteria, interfaceId);
+            }).catch(this.decorateError);
+    }
+
+    getSummaryForTopNConversations(N, start, end, includeOther, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getSummaryForTopNConversations({
+                    N: N,
+                    start: start,
+                    end: end,
+                    exporterNode: nodeCriteria,
+                    ifIndex: interfaceId,
+                    includeOther: includeOther
+                });
+            }).catch(this.decorateError);
+    }
+
+    getSummaryForConversations(conversations, start, end, includeOther, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getSummaryForConversations(conversations, start, end, includeOther, nodeCriteria, interfaceId);
+            }).catch(this.decorateError);
+    }
+
+    getHosts(prefix, start, end, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getHosts(prefix + '.*', start, end, nodeCriteria, interfaceId);
+            }).catch(this.decorateError);
+    }
+
+    getSeriesForHosts(hosts, start, end, step, includeOther, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getSeriesForHosts(hosts, start, end, step, includeOther, nodeCriteria, interfaceId);
+            }).catch(this.decorateError);
+    }
+
+    getSeriesForTopNHosts(N, start, end, step, includeOther, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getSeriesForTopNHosts(N, start, end, step, includeOther, nodeCriteria, interfaceId);
+            }).catch(this.decorateError);
+    }
+
+    getSummaryForTopNHosts(N, start, end, includeOther, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getSummaryForTopNHosts(N, start, end, includeOther, nodeCriteria, interfaceId);
+            }).catch(this.decorateError);
+    }
+
+    getSummaryForHosts(hosts, start, end, includeOther, nodeCriteria, interfaceId) {
+        return this.getFlowDao()
+            .then(function(flowDao) {
+                return flowDao.getSummaryForHosts(hosts, start, end, includeOther, nodeCriteria, interfaceId);
             }).catch(this.decorateError);
     }
 
@@ -301,4 +445,4 @@ export class ClientDelegate {
                 return flowDao.getExporter(nodeCriteria, searchLimit);
             }).catch(this.decorateError);
     }
-}
\ No newline at end of file
+}
