@@ -4,11 +4,10 @@ import $ from "jquery";
 import "jquery.flot";
 import "jquery.flot.selection";
 import "jquery.flot.crosshair";
-import "../../jquery.flot.categories";
-
+import "flot/jquery.flot.categories";
 
 class AlarmHistogramCtrl extends MetricsPanelCtrl {
-
+    /** @ngInject */
     constructor($scope, $injector, $timeout) {
         super($scope, $injector);
 
@@ -199,6 +198,7 @@ class AlarmHistogramCtrl extends MetricsPanelCtrl {
                     });
                 break;
         }
+        this.ctrl.renderingCompleted();
     }
 
     query(data, column) {
