@@ -1,17 +1,25 @@
 import _ from 'lodash';
 
-import kbn from 'app/core/utils/kbn';
+import { grafanaResource } from '../../lib/grafana_resource';
+const getValueFormats = grafanaResource('getValueFormats');
+
+import { defaultColors } from './module';
+
+export const alignTypesEnum = [
+  { text: 'auto', value: '' },
+  { text: 'left', value: 'left' },
+  { text: 'center', value: 'center' },
+  { text: 'right', value: 'right' },
+];
 
 export class ColumnOptionsCtrl {
   /** @ngInject */
-  constructor($scope, $q, uiSegmentSrv) {
-    this.$q = $q;
-    this.uiSegmentSrv = uiSegmentSrv;
+  constructor($scope) {
     $scope.editor = this;
     this.activeStyleIndex = 0;
     this.panelCtrl = $scope.ctrl;
     this.panel = this.panelCtrl.panel;
-    this.unitFormats = kbn.getUnitFormats();
+    this.unitFormats = getValueFormats();
     this.colorModes = [
       {text: 'Disabled', value: null},
       {text: 'Cell', value: 'cell'},
@@ -19,6 +27,7 @@ export class ColumnOptionsCtrl {
       {text: 'Row', value: 'row'},
     ];
     this.columnTypes = [
+      {text: 'Checkbox', value: 'checkbox'},
       {text: 'Number', value: 'number'},
       {text: 'String', value: 'string'},
       {text: 'Date', value: 'date'},
@@ -29,20 +38,29 @@ export class ColumnOptionsCtrl {
     this.dateFormats = [
       {text: 'DD MMM HH:mm:ss', value: 'DD MMM HH:mm:ss'},
       {text: 'YYYY-MM-DD HH:mm:ss', value: 'YYYY-MM-DD HH:mm:ss'},
+      {text: 'YYYY-MM-DD HH:mm:ss.SSS', value: 'YYYY-MM-DD HH:mm:ss.SSS'},
       {text: 'MM/DD/YY h:mm:ss a', value: 'MM/DD/YY h:mm:ss a'},
       {text: 'MMMM D, YYYY LT', value: 'MMMM D, YYYY LT'},
+      {text: 'YYYY-MM-DD', value: 'YYYY-MM-DD'},
       {text: 'relative', value: 'relative'},
       {text: 'relative (short)', value: 'relative-short'}
     ];
+    this.mappingTypes = [
+      { text: 'Value to text', value: 1 },
+      { text: 'Range to text', value: 2 },
+    ];
+    this.alignTypes = _.cloneDeep(alignTypesEnum);
 
     this.getColumnNames = () => {
       if (!this.panelCtrl.table) {
         return [];
       }
-      return _.map(this.panelCtrl.table.columns, function (col) {
+      return _.map(this.panelCtrl.table.columns, (col) => {
         return col.text;
       });
     };
+
+    this.onColorChange = this.onColorChange.bind(this);
   }
 
   render() {
@@ -55,27 +73,29 @@ export class ColumnOptionsCtrl {
   }
 
   addColumnStyle() {
-    let newStyleRule = {
+    const newStyleRule = {
       unit: 'short',
       type: 'number',
       alias: '',
       decimals: 2,
-      colors: ["rgba(245, 54, 54, 0.9)", "rgba(237, 129, 40, 0.89)", "rgba(50, 172, 45, 0.97)"],
+      colors: Array.prototype.concat([ ], defaultColors),
       colorMode: null,
       pattern: '',
       dateFormat: 'YYYY-MM-DD HH:mm:ss',
       thresholds: [],
+      mappingType: 1,
+      align: 'auto',
       width: "",
       clip: false
     };
 
-    let styles = this.panel.styles;
-    let stylesCount = styles.length;
+    const styles = this.panel.styles;
+    const stylesCount = styles.length;
     let indexToInsert = stylesCount;
 
     // check if last is a catch all rule, then add it before that one
     if (stylesCount > 0) {
-      let last = styles[stylesCount - 1];
+      const last = styles[stylesCount - 1];
       if (last.pattern === '/.*/') {
         indexToInsert = stylesCount - 1;
       }
@@ -90,12 +110,45 @@ export class ColumnOptionsCtrl {
   }
 
   invertColorOrder(index) {
-    let ref = this.panel.styles[index].colors;
-    let copy = ref[0];
+    const ref = this.panel.styles[index].colors;
+    const copy = ref[0];
     ref[0] = ref[2];
     ref[2] = copy;
     this.panelCtrl.render();
   }
+
+  onColorChange(style, colorIndex) {
+    return (newColor) => {
+      style.colors[colorIndex] = newColor;
+      this.render();
+    };
+  }
+
+  addValueMap(style) {
+    if (!style.valueMaps) {
+      style.valueMaps = [];
+    }
+    style.valueMaps.push({ value: '', text: '' });
+    this.panelCtrl.render();
+  }
+
+  removeValueMap(style, index) {
+    style.valueMaps.splice(index, 1);
+    this.panelCtrl.render();
+  }
+
+  addRangeMap(style) {
+    if (!style.rangeMaps) {
+      style.rangeMaps = [];
+    }
+    style.rangeMaps.push({ from: '', to: '', text: '' });
+    this.panelCtrl.render();
+  }
+
+  removeRangeMap(style, index) {
+    style.rangeMaps.splice(index, 1);
+    this.panelCtrl.render();
+  }
 }
 
 /** @ngInject */
