@@ -14,17 +14,51 @@ export class TablePanelEditorCtrl {
     this.transformers = transformers;
     this.fontSizes = ['80%', '90%', '100%', '110%', '120%', '130%', '150%', '160%', '180%', '200%', '220%', '250%'];
 
+    this.themes = {
+      helm: 'Helm Default',
+      opennms: 'OpenNMS',
+      omi: 'Oh My!',
+      nnmi: 'No, Never Mind (i)',
+      netcool: "That's Cool",
+    };
+
+    if (!this.themes[this.panel.theme]) {
+      this.panel.theme = 'helm';
+    }
+
     this.srcIndex = undefined;
     this.destIndex = undefined;
 
     this.addColumnSegment = uiSegmentSrv.newPlusButton();
-    let editor = document.querySelectorAll('.editor-row')[0];
+    this.updateTransformHints();
+
+    const editor = document.querySelectorAll('.editor-row')[0];
     for (const e of [ 'dragstart', 'dragover', 'dragleave', 'drop']) {
       //console.log('adding listener: ' + e);
       editor.addEventListener(e, (evt) => { this.handleEvent(e, evt); }, false);
     }
   }
 
+  updateTransformHints() {
+    // BMR: alarm table can always choose which columns to include, and in what order
+    this.canSetColumns = true;
+    this.columnsHelpMessage = '';
+
+    switch (this.panel.transform) {
+      case 'timeseries_aggregations': {
+        this.canSetColumns = true;
+        break;
+      }
+      case 'json': {
+        this.canSetColumns = true;
+        break;
+      }
+      case 'table': {
+        this.columnsHelpMessage = 'Columns and their order are determined by the data query';
+      }
+    }
+  }
+
   removeClasses(...classes) {
     for (const c of classes) {
       const cols = document.querySelectorAll('.' + c);
@@ -64,7 +98,7 @@ export class TablePanelEditorCtrl {
         }
         if (id) {
           this.srcIndex = parseInt(id.replace(/^column-/, ''), 10);
-          console.log('picking up "' + this.panel.columns[this.srcIndex].text + '"');
+          console.log(`picking up "${this.panel.columns[this.srcIndex].text}"`);
         }
         break;
       case 'dragover':
@@ -75,7 +109,7 @@ export class TablePanelEditorCtrl {
         if (target && target.id && target.classList && target.classList.contains('column-reorder')) {
           const columnIndex = parseInt(target.id.replace(/^column-/, ''), 10);
           if (!target.classList.contains('over')) {
-            //console.log('entering ' + this.panel.columns[columnIndex].text);
+            //console.log(`entering ${this.panel.columns[columnIndex].text}`);
             this.removeClasses('over');
             target.classList.add('over');
             this.destIndex = columnIndex;
@@ -85,7 +119,7 @@ export class TablePanelEditorCtrl {
       case 'dragleave':
         if (target && evt.screenX !== 0 && evt.screenY !== 0) {
           //const columnIndex = parseInt(target.id.replace(/^column-/, ''), 10);
-          //console.log('leaving ' + this.panel.columns[columnIndex].text);
+          //console.log(`leaving ${this.panel.columns[columnIndex].text}`);
           this.destIndex = undefined;
           this.removeClasses('over');
         }
@@ -99,7 +133,7 @@ export class TablePanelEditorCtrl {
             this.panel.columns.splice(this.destIndex, 0, this.panel.columns.splice(this.srcIndex, 1)[0]);
             this.panelCtrl.render();
           });
-          console.log('dropped "' + this.panel.columns[this.srcIndex].text + '" onto "' + this.panel.columns[this.destIndex].text + '"');
+          console.log(`dropped "${this.panel.columns[this.srcIndex].text}" onto "${this.panel.columns[this.destIndex].text}"`);
         } else {
           const targetIndex = (this.srcIndex == undefined) ? 'source' : 'destination';
           console.log(`WARNING: drop event received but ${targetIndex} was unset.`);
@@ -107,7 +141,7 @@ export class TablePanelEditorCtrl {
         this.removeClasses('over', 'picked-up');
         return false;
       default:
-        console.log('WARNING: unhandled event type: ' + type);
+        console.log(`WARNING: unhandled event type: ${type}`);
     }
   }
 
@@ -117,22 +151,22 @@ export class TablePanelEditorCtrl {
     }
     let columns = this.transformers[this.panel.transform].getColumns(this.panelCtrl.dataRaw);
     // Filter out columns that have already been selected
-    let self = this;
-    columns = columns.filter(function(a){return self.panel.columns.indexOf(a) < 0});
-    let segments = _.map(columns, (c) => this.uiSegmentSrv.newSegment({value: c.text}));
+    columns = columns.filter(a => this.panel.columns.indexOf(a) < 0);
+
+    const segments = _.map(columns, (c) => this.uiSegmentSrv.newSegment({value: c.text}));
     return this.$q.when(segments);
   }
 
   addColumn() {
-    let columns = transformers[this.panel.transform].getColumns(this.panelCtrl.dataRaw);
-    let column = _.find(columns, {text: this.addColumnSegment.value});
+    const columns = transformers[this.panel.transform].getColumns(this.panelCtrl.dataRaw);
+    const column = _.find(columns, {text: this.addColumnSegment.value});
 
     if (column) {
       this.panel.columns.push(column);
       this.render();
     }
 
-    let plusButton = this.uiSegmentSrv.newPlusButton();
+    const plusButton = this.uiSegmentSrv.newPlusButton();
     this.addColumnSegment.html = plusButton.html;
     this.addColumnSegment.value = plusButton.value;
   }
@@ -143,6 +177,7 @@ export class TablePanelEditorCtrl {
       this.panel.columns.push({text: 'Avg', value: 'avg'});
     }
 
+    this.updateTransformHints();
     this.render();
   }
 
@@ -162,7 +197,7 @@ export function tablePanelEditor($q, uiSegmentSrv) { // eslint-disable-line no-u
   return {
     restrict: 'E',
     scope: true,
-    templateUrl: '/public/plugins/opennms-helm-app/panels/alarm-table/editor.html',
+    templateUrl: 'public/plugins/opennms-helm-app/panels/alarm-table/editor.html',
     controller: TablePanelEditorCtrl,
   };
 }
