@@ -1,6 +1,7 @@
 import _ from 'lodash';
 import $ from 'jquery';
 import {MetricsPanelCtrl} from 'app/plugins/sdk';
+import config from 'app/core/config';
 import {transformDataToTable} from './transformers';
 import {tablePanelEditor} from './editor';
 import {columnOptionsTab} from './column_options';
@@ -13,16 +14,104 @@ import {loadPluginCss} from 'app/plugins/sdk';
 import {SelectionMgr} from "./selection_mgr";
 import {ActionMgr} from "./action_mgr";
 
+import { grafanaResource } from '../../lib/grafana_resource';
+const PanelEvents = grafanaResource('PanelEvents');
+
+import moment from 'moment';
+import * as XLSX from 'xlsx';
+
 loadPluginCss({
   dark: 'plugins/opennms-helm-app/panels/alarm-table/css/table.dark.css',
   light: 'plugins/opennms-helm-app/panels/alarm-table/css/table.light.css'
 });
 
+export const defaultColors = ['rgba(245, 54, 54, 0.9)', 'rgba(237, 129, 40, 0.89)', 'rgba(50, 172, 45, 0.97)'];
 const doubleClickDelay = 250;
 
-class AlarmTableCtrl extends MetricsPanelCtrl {
+// replace this with `import {isTableData} from '@grafana/data'` if we end up requiring grafana 6.3+
+const isTableData = (data) => data && data.hasOwnProperty('columns');
 
-  constructor($scope, $injector, $rootScope, annotationsSrv, $sanitize, $compile, backendSrv, datasourceSrv, timeSrv) {
+const styles = {
+  ack: {
+    type: 'checkbox',
+    alias: 'Ack',
+    pattern: 'Is Acknowledged',
+    align: 'center',
+    width: '2em',
+    clip: true,
+  },
+  count: {
+    unit: 'short',
+    type: 'number',
+    decimals: 0,
+    colors: Array.prototype.concat([], defaultColors),
+    colorMode: null,
+    pattern: '/Count/',
+    align: 'right',
+    width: '3em',
+  },
+  dateTime: {
+    type: 'date',
+    pattern: '/(.*Time|Suppressed Until)/', // Render all Time columns as date, e.g. "Last Event Time", "First Event Time", etc.
+    dateFormat: 'YYYY-MM-DD HH:mm:ss',
+    align: 'center',
+    width: '6em',
+  },
+  defaultString: {
+    type: 'string',
+    pattern: '/.*/',
+  },
+  description: {
+    type: 'string',
+    pattern: 'Description',
+    sanitize: true,
+  },
+  id: {
+    type: 'string',
+    pattern: '/.*ID/', // Render all "* ID" columns as string, otherwise ID 1000 appears as 1.0 K
+  },
+  lastEventTime: {
+    type: 'date',
+    pattern: '/Last Event Time/',
+    dateFormat: 'YYYY-MM-DD HH:mm:ss',
+    align: 'center',
+    width: '5em',
+    alias: 'Last Occurrence',
+    clip: true,
+  },
+  location: {
+    type: 'string',
+    pattern: 'Location',
+    clip: true,
+    width: '5em',
+  },
+  logMessage: {
+    type: 'string',
+    pattern: 'Log Message',
+    alias: 'Message',
+    sanitize: true,
+    clip: true,
+    width: '30em',
+  },
+  node: {
+    type: 'string',
+    pattern: 'Node Label',
+    alias: 'Node',
+    width: '12em',
+  },
+  severity: {
+    type: 'severity',
+    pattern: 'Severity',
+    displayAs: 'icon',
+    align: 'center',
+    clip: true,
+    width: '3em',
+  },
+};
+
+class AlarmTableCtrl extends MetricsPanelCtrl {
+  /** @ngInject */
+  constructor($scope, $injector, $rootScope, annotationsSrv, $sanitize, $compile, backendSrv, datasourceSrv, templateSrv, timeSrv, variableSrv) {
     super($scope, $injector);
     this.$rootScope = $rootScope;
     this.annotationsSrv = annotationsSrv;
@@ -30,72 +119,70 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
     this.$compile = $compile;
     this.backendSrv = backendSrv;
     this.datasourceSrv = datasourceSrv;
+    this.templateSrv = templateSrv;
     this.timeSrv = timeSrv;
+    this.variableSrv = variableSrv;
 
     let panelDefaults = {
       targets: [{}],
       transform: 'table',
-      pageSize: 5,
+      pageSize: 10,
+      pagingPausesRefresh: false,
       showHeader: true,
       styles: [
-        {
-          type: 'severity',
-          pattern: 'Severity',
-          displayAs: 'icon',
-        },
-        {
-          type: 'date',
-          pattern: '/.*Time/', // Render all "* Time" columns as date, e.g. "Last Event Time", "First Event Time", etc.
-          dateFormat: 'YYYY-MM-DD HH:mm:ss',
-        },
-        {
-          type: 'date',
-          pattern: 'Suppressed Until',
-          dateFormat: 'YYYY-MM-DD HH:mm:ss',
-        },
-        {
-          type: 'string',
-          pattern: '/.*ID/', // Render all "* ID" columns as string, otherwise ID 1000 appears as 1.0 K
-        },
-        {
-          type: 'string',
-          pattern: 'Description',
-          sanitize: true
-        },
-        {
-          type: 'string',
-          pattern: 'Log Message',
-          sanitize: true
-        },
-        {
-          unit: 'short',
-          type: 'number',
-          decimals: 0,
-          pattern: 'Count',
-        },
-        {
-          unit: 'short',
-          type: 'number',
-          alias: '',
-          decimals: 2,
-          colors: ["rgba(245, 54, 54, 0.9)", "rgba(237, 129, 40, 0.89)", "rgba(50, 172, 45, 0.97)"],
-          colorMode: null,
-          pattern: '/.*/',
-          thresholds: [],
-        }
+        styles.severity,
+        styles.lastEventTime,
+        styles.dateTime,
+        styles.id,
+        styles.description,
+        styles.logMessage,
+        styles.count,
+        styles.node,
+        styles.ack,
+        styles.location,
+        styles.defaultString,
       ],
       columns: [
-          {text: 'Severity'},
-          {text: 'UEI'},
-          {text: 'Log Message'},
-          {text: 'Node Label'},
-          {text: 'Count'},
-          {text: 'Last Event Time'},
+          {
+            title: 'Ack',
+            text: 'Is Acknowledged',
+            style: styles.ack,
+          },
+          {
+            title: 'Severity',
+            text: 'Severity',
+            style: styles.severity,
+          },
+          {
+            title: 'Count',
+            text: 'Count',
+            style: styles.count,
+          },
+          {
+            title: 'Last Occurrence',
+            text: 'Last Event Time',
+            style: styles.lastEventTime,
+          },
+          {
+            title: 'Location',
+            text: 'Location',
+            style: styles.location,
+          },
+          {
+            title: 'Node',
+            text: 'Node Label',
+            style: styles.node,
+          },
+          {
+            title: 'Message',
+            text: 'Log Message',
+            style: styles.logMessage,
+          },
         ],
       scroll: false, // disable scrolling as the actions popup is not working properly otherwise
       fontSize: '100%',
-      sort: {col: 0, desc: true},
-      severity: true
+      sort: {col: 3, desc: true},
+      severity: 'column',
     };
 
     this.pageIndex = 0;
@@ -109,12 +196,26 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
 
     _.defaults(this.panel, panelDefaults);
 
+    if (this.panel.styles) {
+      this.panel.styles.forEach((style) => {
+        if (style.type === 'number') {
+          if (!style.colors) {
+            style.colors = Array.prototype.concat([], defaultColors);
+          }
+          if (style.colorMode === undefined) {
+            style.colorMode = null;
+          }
+        }
+      });
+    }
+
     let self = this;
     this.selectionMgr = new SelectionMgr((from,to) => self.getRowsInRange(from,to), () => self.render());
-    this.events.on('data-received', this.onDataReceived.bind(this));
-    this.events.on('data-error', this.onDataError.bind(this));
-    this.events.on('data-snapshot-load', this.onDataReceived.bind(this));
-    this.events.on('init-edit-mode', this.onInitEditMode.bind(this));
+    this.events.on(PanelEvents ? PanelEvents.dataReceived        : 'data-received', this.onDataReceived.bind(this));
+    this.events.on(PanelEvents ? PanelEvents.dataError           : 'data-error', this.onDataError.bind(this));
+    this.events.on(PanelEvents ? PanelEvents.dataSnapshotLoad    : 'data-snapshot-load', this.onDataReceived.bind(this));
+    this.events.on(PanelEvents ? PanelEvents.editModeInitialized : 'init-edit-mode', this.onInitEditMode.bind(this));
+    this.events.on(PanelEvents ? PanelEvents.initPanelActions    : 'init-panel-actions', this.onInitPanelActions.bind(this));
 
     if (this.panel.severity === true) {
       this.panel.severity = 'row';
@@ -153,7 +254,7 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
 
   refreshAppConfig() {
     const self = this;
-    self.backendSrv.get(`/api/plugins/opennms-helm-app/settings`).then(result => {
+    self.backendSrv.get('api/plugins/opennms-helm-app/settings').then(result => {
       if (result && result.jsonData) {
         self.appConfig = result.jsonData;
       } else {
@@ -167,12 +268,20 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
     this.addEditorTab('Column Styles', columnOptionsTab, 3);
   }
 
-  issueQueries(datasource) {
-    this.pageIndex = 0;
+  onInitPanelActions(actions) {
+    actions.push({ text: 'Export CSV', click: 'ctrl.exportCSV()' });
+    actions.push({ text: 'Export Excel', click: 'ctrl.exportExcel()' })
+  }
 
+  issueQueries(datasource) {
     if (this.panel.transform === 'annotations') {
-      this.setTimeQueryStart();
-      return this.annotationsSrv.getAnnotations({dashboard: this.dashboard, panel: this.panel, range: this.range})
+      if (this.setTimeQueryStart) this.setTimeQueryStart();
+      return this.annotationsSrv
+        .getAnnotations({
+          dashboard: this.dashboard,
+          panel: this.panel,
+          range: this.range,
+        })
         .then(annotations => {
           return {data: annotations};
         });
@@ -187,36 +296,96 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
   }
 
   onDataReceived(dataList) {
-    this.dataRaw = dataList;
-    this.pageIndex = 0;
+    this.newDataRaw = dataList;
+
+    if (!this.panel.pagingPausesRefresh || this.pageIndex === 0 || !this.dataRaw || this.dataRaw.length === 0) {
+      this.updateData();
+    }
+
+    this.render();
+  }
+
+  refreshData() {
+    this.updateData();
+    this.scope.$evalAsync(() => {
+      this.render();
+    });
+  }
+
+  updateData() {
+    this.dataRaw = this.newDataRaw;
+    delete this.newDataRaw;
 
     // automatically correct transform mode based on data
     if (this.dataRaw && this.dataRaw.length) {
-      if (this.dataRaw[0].type === 'table') {
+      if (isTableData(this.dataRaw[0])) {
         this.panel.transform = 'table';
       } else {
         if (this.dataRaw[0].type === 'docs') {
           this.panel.transform = 'json';
         } else {
           if (this.panel.transform === 'table' || this.panel.transform === 'json') {
-            this.panel.transform = 'timeseries_to_rows';
+            this.panel.transform = 'timeseries_to_rows'; // we had overridden this to 'table', not sure why?
           }
         }
       }
     }
-
-    this.render();
+    this.pageIndex = 0;
   }
 
   render() {
     this.table = transformDataToTable(this.dataRaw, this.panel);
     this.table.sort(this.panel.sort);
 
-    this.renderer = new TableRenderer(this.panel, this.table, this.dashboard.isTimezoneUtc(), this.$sanitize, this.selectionMgr);
+    this.renderer = new TableRenderer(
+      this.panel,
+      this.table,
+      this.dashboard.isTimezoneUtc(),
+      this.$sanitize,
+      this.selectionMgr,
+      this.templateSrv,
+      config.theme.type,
+    );
 
     return super.render(this.table);
   }
 
+  _getWorkbook() {
+    const workbook = XLSX.utils.book_new();
+    workbook.Props = {
+      Title: this.panel.title,
+    }
+
+    const columns = this.table.columns.map((col) => {
+      return col.text;
+    });
+    const rows = this.table.rows.map((row) => {
+      const ret = {};
+      row.forEach((col, index) => {
+        if (moment.isMoment(col) || col instanceof Date) {
+          ret[columns[index]] = moment(col).format('YYYY-MM-DD HH:mm:ss.SSS');
+        } else {
+          ret[columns[index]] = col;
+        }
+      });
+      return ret;
+    });
+
+    const worksheet = XLSX.utils.json_to_sheet(rows, {header:columns});
+    XLSX.utils.book_append_sheet(workbook, worksheet, this.panel.title);
+    return workbook;
+  }
+
+  exportCSV() {
+    const workbook = this._getWorkbook();
+    XLSX.writeFile(workbook, this.panel.title + '.csv');
+  }
+
+  exportExcel() {
+    const workbook = this._getWorkbook();
+    XLSX.writeFile(workbook, this.panel.title + '.xlsx');
+  }
+
   toggleColumnSort(col, colIndex) {
     // remove sort flag from current column
     if (this.table.columns[this.panel.sort.col]) {
@@ -238,7 +407,7 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
 
   link(scope, elem, attrs, ctrl) {
     let data;
-    let panel = ctrl.panel;
+    const panel = ctrl.panel;
     let pageCount = 0;
 
     scope.getColumnStyle = (col) => {
@@ -279,42 +448,51 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
     }
 
     function switchPage(e) {
-      let el = $(e.currentTarget);
-      ctrl.pageIndex = (parseInt(el.text(), 10) - 1);
+      const el = $(e.currentTarget);
+      ctrl.pageIndex = parseInt(el.text(), 10) - 1;
       renderPanel();
     }
 
     function appendPaginationControls(footerElem) {
       footerElem.empty();
 
-      let pageSize = panel.pageSize || 100;
+      const pageSize = panel.pageSize || 100;
       pageCount = Math.ceil(data.rows.length / pageSize);
       if (pageCount === 1) {
         return;
       }
 
-      let startPage = Math.max(ctrl.pageIndex - 3, 0);
-      let endPage = Math.min(pageCount, startPage + 9);
+      const startPage = Math.max(ctrl.pageIndex - 3, 0);
+      const endPage = Math.min(pageCount, startPage + 9);
 
-      let paginationList = $('<ul></ul>');
+      const paginationList = $('<ul></ul>');
 
       for (let i = startPage; i < endPage; i++) {
-        let activeClass = i === ctrl.pageIndex ? 'active' : '';
-        let pageLinkElem = $('<li><a class="table-panel-page-link pointer ' + activeClass + '">' + (i + 1) + '</a></li>');
+        const activeClass = i === ctrl.pageIndex ? 'active' : '';
+        const pageLinkElem = $(
+          '<li><a class="table-panel-page-link pointer ' + activeClass + '">' + (i + 1) + '</a></li>'
+        );
         paginationList.append(pageLinkElem);
       }
 
+      if (ctrl.newDataRaw) {
+        paginationList.append($('<li><a class="table-panel-page-refresh pointer" name="refresh"><i class="fa fa-refresh" aria-hidden="true"></i> refresh</a></li>'));
+      }
       footerElem.append(paginationList);
     }
 
+    function refreshData() {
+      ctrl.refreshData();
+    }
+  
     function renderPanel() {
-      let panelElem = elem.parents('.panel');
-      let rootElem = elem.find('.table-panel-scroll');
-      let tbodyElem = elem.find('tbody');
-      let footerElem = elem.find('.table-panel-footer');
+      const panelElem = elem.parents('.panel-content');
+      const rootElem = elem.find('.table-panel-scroll');
+      const tbodyElem = elem.find('tbody');
+      const footerElem = elem.find('.table-panel-footer .pagination');
 
       elem.css({'font-size': panel.fontSize});
-      panelElem.addClass('table-panel-wrapper');
+      panelElem.addClass('table-panel-content');
 
       appendTableRows(tbodyElem);
       appendPaginationControls(footerElem);
@@ -322,14 +500,35 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
       rootElem.css({'max-height': panel.scroll ? getTableHeight() : ''});
     }
 
-    elem.on('click', '.table-panel-page-link', switchPage);
+    // hook up link tooltips
+    elem.tooltip({
+      selector: '[data-link-tooltip]',
+    });
 
-    let unbindDestroy = scope.$on('$destroy', function () {
+    function addFilterClicked(e) {
+      const filterData = $(e.currentTarget).data();
+      const options = {
+        datasource: panel.datasource,
+        key: data.columns[filterData.column].text,
+        value: data.rows[filterData.row][filterData.column],
+        operator: filterData.operator,
+      };
+
+      ctrl.variableSrv.setAdhocFilter(options);
+    }
+
+    elem.on('click', '.table-panel-page-link', switchPage);
+    elem.on('click', '.table-panel-filter-link', addFilterClicked);
+    elem.on('click', '.table-panel-page-refresh', refreshData);
+
+    const unbindDestroy = scope.$on('$destroy', function () {
       elem.off('click', '.table-panel-page-link');
+      elem.off('click', '.table-panel-filter-link');
+      elem.off('click', '.table-panel-page-refresh');
       unbindDestroy();
     });
 
-    ctrl.events.on('render', function (renderData) {
+    ctrl.events.on(PanelEvents ? PanelEvents.render : 'render', renderData => {
       data = renderData || data;
       if (data) {
         renderPanel();
@@ -340,17 +539,28 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
 
   // Alarm related actions
 
-  findTableRow(source, alarmId) {
+  findRowAndMeta(source, alarmId) {
     let matchedRow;
+    let matchedMeta;
     _.each(this.dataRaw, table => {
-      let filteredRow = _.find(table.rows, row => {
-        return row.meta.source === source && row.meta.alarm.id === alarmId;
-      });
-      if (filteredRow !== undefined) {
-        matchedRow = filteredRow;
+      if (table.meta && table.meta.entity_metadata) {
+        const rowIdx = _.findIndex(table.meta.entity_metadata, meta => {
+          return meta.source === source && meta.alarm.id === alarmId;
+        });
+        const filteredRow = table.rows[rowIdx];
+        if (filteredRow !== undefined) {
+          matchedRow = filteredRow;
+          matchedMeta = table.meta.entity_metadata[rowIdx];
+        }
       }
     });
-    return matchedRow;
+    if (matchedRow) {
+      return {
+        row: matchedRow,
+        meta: matchedMeta,
+      };
+    }
+    return null;
   }
 
   getContextMenu($event, source, alarmId) {
@@ -370,10 +580,10 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
     selectedRows = _.map(selectedRows, row => {
       // Create new objects instead of modifying the existing rows
       // returned by SelectionMgr#getSelectedRows()
-      const tableRow = self.findTableRow(row.source, row.alarmId);
+      const tableRow = self.findRowAndMeta(row.source, row.alarmId);
       return Object.assign({}, row, {
-        alarm: tableRow ? tableRow.meta.alarm : undefined,
-        ticketerConfig: tableRow ? tableRow.meta.ticketerConfig : undefined
+        alarm: tableRow && tableRow.meta ? tableRow.meta.alarm : undefined,
+        ticketerConfig: tableRow && tableRow.meta ? tableRow.meta.ticketerConfig : undefined
       });
     });
 
@@ -459,8 +669,8 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
   }
 
   alarmDetails(source, alarmId) {
-    const row = this.findTableRow(source, alarmId);
-    if (row === undefined) {
+    const row = this.findRowAndMeta(source, alarmId);
+    if (!row) {
       this.$rootScope.appEvent('alert-error', ['Unable to find matching alarm', '']);
       return;
     }
@@ -468,6 +678,7 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
     let newScope = this.$rootScope.$new();
     newScope.severity = this.panel.severity;
     newScope.source = source;
+    newScope.theme = this.panel.theme;
     newScope.alarm = row.meta.alarm;
     newScope.ticketerConfig = row.meta.ticketerConfig;
 
@@ -479,39 +690,50 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
 
   performAlarmActionOnDatasource(source, action, alarmId) {
     let self = this;
-    this.datasourceSrv.get(source).then(ds => {
-      if (ds.type && ds.type.indexOf("fault-datasource") < 0) {
-        throw {message: 'Only OpenNMS datasources are supported'};
-      } else {
-        if (!ds[action]) {
-          throw {message: 'Action ' + action + ' not implemented by datasource ' + ds.name + " of type " + ds.type};
+    
+    return new Promise((resolve, reject) => {
+      this.datasourceSrv.get(source).then(ds => {
+        if (ds.type && ds.type.indexOf("entity-datasource") < 0) {
+          throw {message: 'Only OpenNMS datasources are supported'};
+        } 
+        else {
+          if (!ds[action]) {
+            throw {message: 'Action ' + action + ' not implemented by datasource ' + ds.name + " of type " + ds.type};
+          }
+
+          var actionPerfomed = ds[action](alarmId);
+          actionPerfomed.then((successObj) => {
+            resolve(successObj);
+          }, (error) => {
+            reject(error);
+          });
         }
-        return ds[action](alarmId);
-      }
-    }).then(() => {
-      // Action was successful, remove any previous error
-      delete self.error;
-      // Refresh the dashboard
-      self.timeSrv.refreshDashboard();
-    }).catch(err => {
-      self.error = err.message || "Request Error";
-    });
+      }).catch(err => {
+        self.error = err.message || "Request Error";
+        reject(err);
+      });
+    })
+  }
+
+  /* Refreshing Dashboard Panel */
+  refreshDashboard(){
+    this.timeSrv.refreshDashboard();
   }
 
   acknowledgeAlarm(source, alarmId) {
-    this.performAlarmActionOnDatasource(source, 'acknowledgeAlarm', alarmId);
+    return this.performAlarmActionOnDatasource(source, 'acknowledgeAlarm', alarmId);
   }
 
   unacknowledgeAlarm(source, alarmId) {
-    this.performAlarmActionOnDatasource(source, 'unacknowledgeAlarm', alarmId);
+    return this.performAlarmActionOnDatasource(source, 'unacknowledgeAlarm', alarmId);
   }
 
   clearAlarm(source, alarmId) {
-    this.performAlarmActionOnDatasource(source, 'clearAlarm', alarmId);
+    return this.performAlarmActionOnDatasource(source, 'clearAlarm', alarmId)
   }
 
   escalateAlarm(source, alarmId) {
-    this.performAlarmActionOnDatasource(source, 'escalateAlarm', alarmId);
+    return this.performAlarmActionOnDatasource(source, 'escalateAlarm', alarmId);
   }
 
   createTicketForAlarm(source, alarmId) {
@@ -534,9 +756,12 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
     }
 
     let findIdx = selectionToMatch => {
-      return _.findIndex(this.table.rows, row => {
-        return row.meta.source === selectionToMatch.source && row.meta.alarm.id === selectionToMatch.alarmId;
-      });
+      if (this.table && this.table.meta && this.table.meta.entity_metadata) {
+        return _.findIndex(this.table.meta.entity_metadata, meta => {
+          return meta.source === selectionToMatch.source && meta.alarm.id === selectionToMatch.alarmId;
+        });
+      }
+      return undefined;
     };
 
     let startIdx = findIdx(from);
@@ -555,9 +780,10 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
     }
 
     for (let i = startIdx; i <= endIdx; i++) {
+      const meta = this.table.meta && this.table.meta.entity_metadata ? this.table.meta.entity_metadata[i] : {};
       rows.push({
-        source: this.table.rows[i].meta.source,
-        alarmId: this.table.rows[i].meta.alarm.id
+        source: meta.source,
+        alarmId: meta.alarm.id
       });
     }
     return rows;
@@ -566,19 +792,25 @@ class AlarmTableCtrl extends MetricsPanelCtrl {
 
 AlarmTableCtrl.templateUrl = 'panels/alarm-table/module.html';
 
-export {
-  AlarmTableCtrl,
-  AlarmTableCtrl as PanelCtrl
-};
+export { AlarmTableCtrl, AlarmTableCtrl as PanelCtrl };
 
 coreModule.directive('alarmDetailsAsModal',  alarmDetailsAsDirective);
 coreModule.directive('memoEditor',  memoEditorAsDirective);
 coreModule.directive('contextMenu', contextMenuAsDirective());
+
 coreModule.directive('dynamicHeight', function($window) {
   // Used to dynamically size the alarm details modal window
   return{
     link: function(scope, element /*, attrs */) {
-      element.css('max-height', $window.innerHeight * 0.8 + 'px');
+      const doResize = () => {
+        element.css('max-height', $window.innerHeight * 0.8 + 'px');
+      };
+
+      doResize();
+      element.on('$destroy', () => {
+        $window.removeEventListener('resize', doResize);
+      });
+      $window.addEventListener('resize', doResize);
     }
   }
 });
