@@ -1,3 +1,4 @@
+/*eslint no-unused-vars: "warn"*/
 import prunk from 'prunk';
 import jsdom from 'jsdom';
 import chai from 'chai';
@@ -13,6 +14,14 @@ prunk.mock('app/plugins/sdk', {
 prunk.mock('app/core/app_events', {
   appEvents: null
 });
+prunk.mock('app/core/utils/kbn', {
+  interval_to_ms: () => {return 0;}
+});
+prunk.mock('angular', {
+  $: {
+    isNumeric: () => {return true;}
+  }
+});
 
 // Setup jsdom
 // Required for loading angularjs
